{*******************************************************}
{                                                       }
{       Turbo Pascal Version 6.0                        }
{       Turbo Vision Unit                               }
{                                                       }
{       Copyright (c) 1990 Borland International        }
{                                                       }
{*******************************************************}
{  Doug Hood, CIS 70324,3336                            }
{    Released to the PUBLIC DOMAIN                      }
{*******************************************************}
{     MODS: added Multiple line capability              }
{           added NoEcho input box                      }
{           added Date  input box                       }
{           added Button_Order_Flag                     }
{            (lists buttons backwards order)            }
{                                                       }
{*******************************************************}

unit MsgBox;

{$F+,O+,X+,D-}

interface

uses Objects;

const

{ Message box classes }

  mfWarning      = $0000;       { Display a Warning box }
  mfError        = $0001;       { Dispaly a Error box }
  mfInformation  = $0002;       { Display an Information Box }
  mfConfirmation = $0003;       { Display a Confirmation Box }

{ Message box button flags }

  mfYesButton    = $0100;       { Put a Yes button into the dialog }
  mfNoButton     = $0200;       { Put a No button into the dialog }
  mfOKButton     = $0400;       { Put an OK button into the dialog }
  mfCancelButton = $0800;       { Put a Cancel button into the dialog }

  mfYesNoCancel  = mfYesButton + mfNoButton + mfCancelButton;
                                { Standard Yes, No, Cancel dialog }
  mfOKCancel     = mfOKButton + mfCancelButton;
                                { Standard OK, Cancel dialog }

{-*************************************************************-}
{ MessageBox displays the given string in a standard sized      }
{ dialog box. Before the dialog is displayed the Msg and Params }
{ are passed to FormatStr.  The resulting string is displayed   }
{ as a TStaticText view in the dialog.                          }
{     (use 'hello'#13'there' to split a string into 2 lines)    }
{-*************************************************************-}

function MessageBox(Msg: String; Params: Pointer; AOptions: Word): Word;
function MessageBox2(Msg1: String; Params1: Pointer;
                     Msg2: String; Params2: Pointer;
                     Msg3: String; Params3: Pointer;
                     Msg4: String; Params4: Pointer;
                     AOptions: Word;
                     Button_Order_Normal : boolean): Word;
function Simple_Message (Msg : string) : Word;


{-*************************************************************-}
{ MessageBoxRec allows the specification of a TRect for the     }
{ message box to occupy.                                        }
{-*************************************************************-}

function MessageBoxRect  (var R: TRect; Msg: String; Params: Pointer;
                          AOptions: Word): Word;
function MessageBoxRect2 (var R: TRect; Msg1: String; Params1: Pointer;
                          Msg2: String; Params2: Pointer;
                          Msg3: String; Params3: Pointer;
                          Msg4: String; Params4: Pointer;
                          AOptions: Word;
                          Button_Order_Normal : boolean): Word;


{-*************************************************************-}
{ InputBox displays a simple dialog that allows the user to     }
{ type in a string.                                             }
{-*************************************************************-}

function InputBox(Title: String; ALabel: String; var S: String;
  Limit: Byte): Word;


{-*************************************************************-}
{ InputBoxRect is like InputBox but allows the specification of }
{ a rectangle.                                                  }
{-*************************************************************-}

function InputBoxRect(var Bounds: TRect; Title: String; ALabel: String;
  var S: String;  Limit: Byte): Word;


{-*************************************************************-}
{ NoEcho_InputBox displays a simple dialog that allows the user }
{ to type in a string, but only '*'s are echoed to the screen.  }
{-*************************************************************-}
function NoEcho_InputBox (Title: String; ALabel: String; var S: String;
                          Limit: Byte;
                          Force_Uppercase : boolean): Word;


{-*************************************************************-}
{ NoEcho_InputBoxRect is like NoEcho_InputBox but allows the    }
{ specification of  a rectangle.                                }
{-*************************************************************-}
function NoEcho_InputBoxRect (var Bounds      : TRect;
                              Title, ALabel   : string;
                              var S           : string;
                              Limit           : byte;
                              Force_Uppercase : boolean) : word;

{-*************************************************************-}
{ Date_InputBox displays a simple dialog that allows the user   }
{ to type in a string, checks for valid date                 .  }
{-*************************************************************-}
function Date_InputBox  (Title: String; ALabel: String; var S: String;
                         Limit: Byte;
                         Default_Today   : boolean;
                         Var Day,Month, Year : word) : word;
{-*************************************************************-}
{ Date_InputBoxRect is like Date_InputBox but allows the        }
{ specification of  a rectangle.                                }
{-*************************************************************-}
function Date_InputBoxRect (var Bounds      : TRect;
                            Title, ALabel   : string;
                            var S           : string;
                            Limit           : byte;
                            Default_Today   : boolean;
                            Var Day,Month, Year : word) : word;

{************************************************************************}
{************************************************************************}
{************************************************************************}
IMPLEMENTATION

uses Drivers, Views, Dialogs, App,
     Fields_Color;  {for noecho_inputline, date_inputline}

{*************************************************************************}
function Simple_Message (Msg : string) : Word;
begin
  Simple_Message := MessageBox2 (Msg, nil, '',nil, '',nil, '',nil,
                                 mfInformation+mfOkButton, TRUE);
end; {simple_message}

{*************************************************************************}
function MessageBox (Msg: String; Params: Pointer;
                     AOptions: Word): Word;
begin
  MessageBox := MessageBox2 (Msg, Params, '',nil, '',nil, '',nil,
                             AOptions, TRUE);
end; {messagebox}

{*************************************************************************}
function MessageBox2(Msg1: String; Params1: Pointer;
                     Msg2: String; Params2: Pointer;
                     Msg3: String; Params3: Pointer;
                     Msg4: String; Params4: Pointer;
                     AOptions: Word;
                     Button_Order_Normal : boolean): Word;
var
  R: TRect;
begin
  R.Assign(0, 0, 40, 9);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  MessageBox2 := MessageBoxRect2 (R, Msg1, Params1,
                                  Msg2, Params2,
                                  Msg3, Params3,
                                  Msg4, Params4,
                                  AOptions,
                                  Button_Order_Normal);
end; {messagebox2}


{*************************************************************************}
function MessageBoxRect (var R: TRect; Msg: String; Params: Pointer;
                         AOptions: Word) : Word;
begin
  MessageBoxRect := MessageBoxRect2 (R, Msg, Params, '',nil, '',nil, '',nil,
                                     AOptions, TRUE);
end; {messageBoxRect}


{*************************************************************************}
function MessageBoxRect2 (var R: TRect; Msg1: String; Params1: Pointer;
                          Msg2: String; Params2: Pointer;
                          Msg3: String; Params3: Pointer;
                          Msg4: String; Params4: Pointer;
                          AOptions: Word;
                          Button_Order_Normal : boolean): Word;
const
  ButtonName: array[0..3] of string[6] =
    ('~Y~es', '~N~o', 'O~K~', 'Cancel');
  Commands: array[0..3] of word =
    (cmYes, cmNo, cmOK, cmCancel);
  Titles: array[0..3] of string[11] =
    ('Warning','Error','Information','Confirm');
var
  I, Idx, X, ButtonCount: Integer;
  Dialog: PDialog;
  Control: PView;
  T: TRect;
  ButtonList: array[0..4] of PView;
  S: String;
  Num_Lines : integer;
  Curr_Pos  : integer;
begin
  Dialog := New(PDialog,
    Init(R, Titles[AOptions and $3]));
  with Dialog^ do
  begin
    Num_Lines := 0;
    R.Assign(2, 2, Size.X - 2, Size.Y - 2);
    FormatStr(S, Msg1, Params1^);
    Control := New(PStaticText, Init(R, S));
    Insert(Control);

    IF (LENGTH(Msg2) > 0) THEN
      BEGIN
        Curr_Pos := 0;
        INC(Num_Lines);
        REPEAT
          Msg1 := COPY(Msg1, Curr_Pos+1, LENGTH(Msg1));
          Curr_Pos := POS (#13, Msg1);
          IF (Curr_Pos > 0)
            THEN INC(Num_Lines);
        UNTIL (Curr_Pos = 0);

        R.Assign(2, 2+Num_Lines, Size.X - 2, Size.Y - 2);
        FormatStr(S, Msg2, Params2^);
        Control := New(PStaticText, Init(R, S));
        Insert(Control);
      END; {if}

    IF (LENGTH(Msg3) > 0) THEN
      BEGIN
        Curr_Pos := 0;
        INC(Num_Lines);
        REPEAT
          Msg2 := COPY(Msg2, Curr_Pos+1, LENGTH(Msg2));
          Curr_Pos := POS (#13, Msg2);
          IF (Curr_Pos > 0)
            THEN INC(Num_Lines);
        UNTIL (Curr_Pos = 0);

        R.Assign(2, 2+Num_Lines, Size.X - 2, Size.Y - 2);
        FormatStr(S, Msg3, Params3^);
        Control := New(PStaticText, Init(R, S));
        Insert(Control);
      END; {if}

    IF (LENGTH(Msg4) > 0) THEN
      BEGIN
        Curr_Pos := 0;
        INC(Num_Lines);
        REPEAT
          Msg3 := COPY(Msg3, Curr_Pos+1, LENGTH(Msg3));
          Curr_Pos := POS (#13, Msg3);
          IF (Curr_Pos > 0)
            THEN INC(Num_Lines);
        UNTIL (Curr_Pos = 0);
        R.Assign(2, 2+Num_Lines, Size.X - 2, Size.Y - 2);
        FormatStr(S, Msg4, Params4^);
        Control := New(PStaticText, Init(R, S));
        Insert(Control);
      END; {if}

    X := -2;
    ButtonCount := 0;
    for I := 0 to 3 do
    begin
      IF (Button_Order_Normal)
        THEN Idx := i
        ELSE Idx := 3-i;
      if AOptions and ($0100 shl Idx) <> 0 then
      begin
        R.Assign(0, 0, 10, 2);
        Control := New(PButton, Init(R, ButtonName[Idx], Commands[idx],
          bfNormal));
        Inc(X, Control^.Size.X + 2);
        ButtonList[ButtonCount] := Control;
        Inc(ButtonCount);
      end;
    end; {for}
    X := (Size.X - X) shr 1;
    for I := 0 to ButtonCount - 1 do
    begin
      Control := ButtonList[I];
      Insert(Control);
      Control^.MoveTo(X, Size.Y - 3);
      Inc(X, Control^.Size.X + 2);
    end;
    SelectNext(False);
  end;
  MessageBoxRect2 := DeskTop^.ExecView(Dialog);
  Dispose(Dialog, Done);
end;

{*************************************************************************}
function InputBox(Title: String; ALabel: String; var S: String;
  Limit: Byte): Word;
var
  R: TRect;
begin
  R.Assign(0, 0, 60, 8);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  InputBox := InputBoxRect(R, Title, ALabel, S, Limit);
end;

{*************************************************************************}
function InputBoxRect(var Bounds: TRect; Title: String; ALabel: String;
  var S: String;  Limit: Byte): Word;
var
  Dialog: PDialog;
  Control: PView;
  R: TRect;
  C: Word;
begin
  Dialog := New(PDialog, Init(Bounds, Title));
  with Dialog^ do
  begin
    R.Assign(4 + CStrLen(ALabel), 2, Size.X - 3, 3);
    Control := New(PInputLine, Init(R, Limit));
    Insert(Control);
    R.Assign(2, 2, 3 + CStrLen(ALabel), 3);
    Insert(New(PLabel, Init(R, ALabel, Control)));
    R.Assign(Size.X - 24, Size.Y - 4, Size.X - 14, Size.Y - 2);
    Insert(New(PButton, Init(R, 'O~K~', cmOk, bfDefault)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    Insert(New(PButton, Init(R, 'Cancel', cmCancel, bfNormal)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    SelectNext(False);
  end;
  Dialog^.SetData(S);
  C := DeskTop^.ExecView(Dialog);
  if C <> cmCancel then Dialog^.GetData(S);
  Dispose(Dialog, Done);
  InputBoxRect := C;
end;

{*************************************************************************}
function NoEcho_InputBox (Title: String; ALabel: String; var S: String;
                          Limit: Byte;
                          Force_Uppercase : boolean): Word;
var
  R: TRect;
begin
  R.Assign(0, 0, 60, 8);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  NoEcho_InputBox := NoEcho_InputBoxRect(R, Title, ALabel, S, Limit,
                                            Force_Uppercase);
end; {noecho_inputbox}

{************************************************************************}
function NoEcho_InputBoxRect (var Bounds      : TRect;
                              Title, ALabel   : string;
                              var S           : string;
                              Limit           : byte;
                              Force_Uppercase : boolean) : word;
var
  Dialog: PDialog;
  Control: PView;
  R: TRect;
  C: Word;
begin
  Dialog := New(PDialog, Init(Bounds, Title));
  with Dialog^ do
  begin
    R.Assign(4 + CStrLen(ALabel), 2, Size.X - 3, 3);
    Control := New(FIELDS_COLOR.PNoEcho_InputLine, Init(R, Limit,0,True,
                                                     Force_Uppercase));
    Insert(Control);
    R.Assign(2, 2, 3 + CStrLen(ALabel), 3);
    Insert(New(PLabel, Init(R, ALabel, Control)));
    R.Assign(Size.X - 24, Size.Y - 4, Size.X - 14, Size.Y - 2);
    Insert(New(PButton, Init(R, 'O~K~', cmOk, bfDefault)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    Insert(New(PButton, Init(R, 'Cancel', cmCancel, bfNormal)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    SelectNext(False);
  end;
  Dialog^.SetData(S);
  C := DeskTop^.ExecView(Dialog);
  if C <> cmCancel then Dialog^.GetData(S);
  Dispose(Dialog, Done);
  NoEcho_InputBoxRect := C;
end; {noecho_inputboxrect}

{-*************************************************************-}
{ Date_InputBox displays a simple dialog that allows the user   }
{ to type in a string, checks for valid date                 .  }
{-*************************************************************-}
function Date_InputBox  (Title: String; ALabel: String; var S: String;
                         Limit: Byte;
                         Default_Today   : boolean;
                         Var Day,Month, Year : word) : word;
var
  R: TRect;
begin
  R.Assign(0, 0, 60, 8);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  Date_InputBox := Date_InputBoxRect(R, Title, ALabel, S, Limit,
                                     Default_Today,
                                     Day,Month,Year);
end; {date_inputbox}

{-*************************************************************-}
{ Date_InputBoxRect is like Date_InputBox but allows the        }
{ specification of  a rectangle.                                }
{-*************************************************************-}
function Date_InputBoxRect (var Bounds      : TRect;
                            Title, ALabel   : string;
                            var S           : string;
                            Limit           : byte;
                            Default_Today   : boolean;
                            Var Day,Month, Year : word) : word;
var
  Dialog: PDialog;
  Date_Field : PDate_InputLine;
  R: TRect;
  C: Word;
begin
  Dialog := New(PDialog, Init(Bounds, Title));
  with Dialog^ do
  begin
    R.Assign(4 + CStrLen(ALabel), 2, Size.X - 3, 3);
    Date_Field := New(FIELDS_COLOR.PDate_InputLine, Init(R, Limit,0,FALSE,
                                                   FALSE,
                                                   Default_Today));
    Insert(Date_Field);
    R.Assign(2, 2, 3 + CStrLen(ALabel), 3);
    Insert(New(PLabel, Init(R, ALabel, Date_Field)));
    R.Assign(Size.X - 24, Size.Y - 4, Size.X - 14, Size.Y - 2);
    Insert(New(PButton, Init(R, 'O~K~', cmOk, bfDefault)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    Insert(New(PButton, Init(R, 'Cancel', cmCancel, bfNormal)));
    Inc(R.A.X, 12); Inc(R.B.X, 12);
    SelectNext(False);
  end;
  IF (LENGTH(S) > 0)
    THEN Dialog^.SetData(S);
  C := DeskTop^.ExecView(Dialog);
  if (C <> cmCancel) then
    BEGIN
      Dialog^.GetData(S);  {the string of the date}
      Day := Date_Field^.Day;
      Month := Date_Field^.Month;
      Year := Date_Field^.Year;
    END;
  Dispose(Dialog, Done);
  Date_InputBoxRect := C;
end; {date_inputboxrect}


end. {unit msgbox}


